# -*- coding: utf-8 -*-

# Copyright (C) 2012  Matias Bordese
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Success result
SUCCESS = 0

#
# Open Modes
#

# headers only
RAR_OM_LIST = 0
# testing and extracting
RAR_OM_EXTRACT = 1
# headers only, include splitted volumes
RAR_OM_LIST_INCSPLIT = 2

#
# Process file operations
#

RAR_SKIP = 0
RAR_TEST = 1
RAR_EXTRACT = 2
RAR_CANCEL_EXTRACT = -1

#
# Errors
#

# End of archive
ERAR_END_ARCHIVE = 10
# Not enough memory to initialize data structures
ERAR_NO_MEMORY = 11
# Archive header broken
ERAR_BAD_DATA = 12
# File is not valid RAR archive
ERAR_BAD_ARCHIVE = 13
# Unknown encryption used for archive headers
ERAR_UNKNOWN_FORMAT = 14
# Archive/Volume open error
ERAR_EOPEN = 15
# File create error
ERAR_ECREATE = 16
# Archive/File close error
ERAR_ECLOSE = 17
# Read error
ERAR_EREAD = 18
# Write error
ERAR_EWRITE = 19
# Buffer too small, comments not completely read
ERAR_SMALL_BUF = 20
# Unknown error
ERAR_UNKNOWN = 21
# Missing password error
ERAR_MISSING_PASSWORD = 22

#
# Comments state
#

# comments not present
RAR_NO_COMMENTS = 0
# Comments read completely
RAR_COMMENTS_SUCCESS = 1


#
# Host OS
#

RAR_DOS = 0
RAR_OS2 = 1
RAR_WIN = 2
RAR_UNIX = 3



# -*- coding: utf-8 -*-

# Copyright (C) 2012  Matias Bordese
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import ctypes
import os
import sys

from unrar import constants
from unrar import unrarlib


__all__ = ["BadRarFile", "is_rarfile", "RarFile", "RarInfo"]


class BadRarFile(Exception):
    """RAR file error."""


def is_rarfile(filename):
    """Return true if file is a valid RAR file."""
    mode = constants.RAR_OM_LIST_INCSPLIT
    archive = unrarlib.RAROpenArchiveDataEx(filename, mode=mode)
    try:
        handle = unrarlib.RAROpenArchiveEx(ctypes.byref(archive))
    except unrarlib.UnrarException:
        return False
    unrarlib.RARCloseArchive(handle)
    return (archive.OpenResult == constants.SUCCESS)


class RarInfo(object):
    """Class with attributes describing each member in the RAR archive."""

    __slots__ = (
            'filename',
            'date_time',
            'compress_type',
            'comment',
            'create_system',
            'extract_version',
            'flag_bits',
            'CRC',
            'compress_size',
            'file_size',
            '_raw_time',
        )

    def __init__(self, header):
        """Initialize a RarInfo object with a member header data."""
        self.filename = header.FileNameW
        self._raw_time = header.FileTime
        self.date_time = unrarlib.dostime_to_timetuple(header.FileTime)
        self.compress_size = header.PackSize + (header.PackSizeHigh << 32)
        self.file_size = header.UnpSize + (header.UnpSizeHigh << 32)
        self.create_system = header.HostOS
        self.extract_version = header.UnpVer
        self.CRC = header.FileCRC
        self.flag_bits = header.Flags
        if header.CmtState == constants.RAR_COMMENTS_SUCCESS:
            self.comment = header.CmtBuf.value
        else:
            self.comment = None


class RarFile(object):
    """RAR archive file."""

    def __init__(self, filename, mode='r', pwd=None):
        """Load RAR archive file with mode read only "r"."""
        self.filename = filename
        mode = constants.RAR_OM_LIST_INCSPLIT

        archive = unrarlib.RAROpenArchiveDataEx(filename, mode=mode)
        handle = self._open(archive)

        # assert(archive.OpenResult == constants.SUCCESS)
        self.pwd = pwd
        self.filelist = []
        self.NameToInfo = {}
        if archive.CmtState == constants.RAR_COMMENTS_SUCCESS:
            self.comment = archive.CmtBuf.value
        else:
            self.comment = None
        self._load_metadata(handle)
        self._close(handle)

    def _read_header(self, handle):
        """Read current member header into a RarInfo object."""
        rarinfo = None
        header_data = unrarlib.RARHeaderDataEx()
        res = unrarlib.RARReadHeaderEx(handle, ctypes.byref(header_data))
        if res != constants.ERAR_END_ARCHIVE:
            rarinfo = RarInfo(header=header_data)
        return rarinfo

    def _process_current(self, handle, op, dest_path=None, dest_name=None):
        """Process current member with 'op' operation."""
        unrarlib.RARProcessFile(handle, op, dest_path, dest_name)

    def _load_metadata(self, handle):
        """Load archive members metadata."""
        rarinfo = self._read_header(handle)
        while rarinfo:
            self.filelist.append(rarinfo)
            self.NameToInfo[rarinfo.filename] = rarinfo
            self._process_current(handle, constants.RAR_SKIP)
            rarinfo = self._read_header(handle)

    def _open(self, archive):
        """Open RAR archive file."""
        try:
            handle = unrarlib.RAROpenArchiveEx(ctypes.byref(archive))
        except unrarlib.UnrarException:
            raise BadRarFile("Invalid RAR file.")
        return handle

    def _close(self, handle):
        """Close RAR archive file."""
        try:
            unrarlib.RARCloseArchive(handle)
        except unrarlib.UnrarException:
            raise BadRarFile("RAR archive close error.")

    def namelist(self):
        """Return a list of file names in the archive."""
        names = []
        for member in self.filelist:
            names.append(member.filename)
        return names

    def setpassword(self, pwd):
        """Set default password for encrypted files."""
        self._pwd = pwd

    def getinfo(self, name):
        """Return the instance of RarInfo given 'name'."""
        rarinfo = self.NameToInfo.get(name)
        if rarinfo is None:
            raise KeyError('There is no item named %r in the archive' % name)
        return rarinfo

    def infolist(self):
        """Return a list of class RarInfo instances for files in the
        archive."""
        return self.filelist

    def printdir(self):
        """Print a table of contents for the RAR file."""
        print "%-46s %19s %12s" % ("File Name", "Modified    ", "Size")
        for rarinfo in self.filelist:
            date = "%d-%02d-%02d %02d:%02d:%02d" % rarinfo.date_time[:6]
            print "%-46s %s %12d" % (rarinfo.filename, date, rarinfo.file_size)

    def testrar(self):
        """Read all the files and check the CRC."""
        error = None
        rarinfo = None
        archive = unrarlib.RAROpenArchiveDataEx(
            self.filename, mode=constants.RAR_OM_EXTRACT)
        handle = self._open(archive)

        try:
            rarinfo = self._read_header(handle)
            while rarinfo is not None:
                self._process_current(handle, constants.RAR_TEST)
                rarinfo = self._read_header(handle)
        except unrarlib.UnrarException:
            error = rarinfo.filename if rarinfo else self.filename
        finally:
            self._close(handle)
        return error

    def extract(self, member, path=None, pwd=None):
        """Extract a member from the archive to the current working directory,
           using its full name. Its file information is extracted as accurately
           as possible. `member' may be a filename or a RarInfo object. You can
           specify a different directory using `path'.
        """
        if isinstance(member, RarInfo):
            member = member.filename

        if path is None:
            path = os.getcwd()

        self._extract_members([member], path, pwd)
        return os.path.join(path, member)

    def extractall(self, path=None, members=None, pwd=None):
        """Extract all members from the archive to the current working
           directory. `path' specifies a different directory to extract to.
           `members' is optional and must be a subset of the list returned
           by namelist().
        """
        if members is None:
            members = self.namelist()
        self._extract_members(members, path, pwd)

    def _extract_members(self, members, targetpath, pwd):
        """Extract the RarInfo objects 'members' to a physical
           file on the path targetpath.
        """
        archive = unrarlib.RAROpenArchiveDataEx(
            self.filename, mode=constants.RAR_OM_EXTRACT)
        handle = self._open(archive)

        password = pwd or self.pwd
        if password is not None:
            unrarlib.RARSetPassword(handle, password)

        try:
            rarinfo = self._read_header(handle)
            while rarinfo is not None:
                if rarinfo.filename in members:
                    self._process_current(
                        handle, constants.RAR_EXTRACT, targetpath)
                else:
                    self._process_current(handle, constants.RAR_SKIP)
                rarinfo = self._read_header(handle)
        except unrarlib.UnrarException:
            raise BadRarFile("Bad RAR archive data.")
        finally:
            self._close(handle)


def main(args=None):
    import textwrap
    USAGE = textwrap.dedent("""\
        Usage:
            rarfile.py -l rarfile.rar        # Show listing of a rarfile
            rarfile.py -t rarfile.rar        # Test if a rarfile is valid
            rarfile.py -e rarfile.rar target # Extract rarfile into target dir
        """)

    valid_args = {'-l': 2, '-e': 3, '-t': 2}
    if args is None:
        args = sys.argv[1:]

    cmd = args and args[0] or None
    if not cmd or cmd not in valid_args or len(args) != valid_args[cmd]:
        print USAGE
        sys.exit(1)

    if cmd == '-l':
        # list
        rf = RarFile(args[1], 'r')
        rf.printdir()
    elif cmd == '-t':
        # test
        rf = RarFile(args[1], 'r')
        err = rf.testrar()
        if err:
            print("The following enclosed file is corrupted: {!r}".format(err))
        print "Done testing"
    elif cmd == '-e':
        # extract
        rf = RarFile(args[1], 'r')
        dest = args[2]
        rf.extractall(path=dest)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Copyright (C) 2012  Matias Bordese
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import tempfile
import unittest

from unrar.rarfile import (
    BadRarFile,
    RarFile,
    RarInfo,
    is_rarfile,
)


TESTS_DIR = os.path.dirname(os.path.realpath(__file__))


class TestIsRarFile(unittest.TestCase):

    def test_rar_israrfile(self):
        rar_filename = os.path.join(TESTS_DIR, 'test_rar.rar')
        self.assertTrue(is_rarfile(rar_filename))

    def test_no_rar_israrfile(self):
        another_file = os.path.join(TESTS_DIR, '__init__.py')
        self.assertFalse(is_rarfile(another_file))

    def test_no_exists_israrfile(self):
        another_file = os.path.join(TESTS_DIR, 'foo.rar')
        self.assertFalse(is_rarfile(another_file))


class TestRarFile(unittest.TestCase):

    def setUp(self):
        super(TestRarFile, self).setUp()
        self.rar = self._open_rarfile()

    def _open_rarfile(self):
        rar_filename = os.path.join(TESTS_DIR, 'test_rar.rar')
        rar = RarFile(rar_filename)
        return rar

    def test_namelist(self):
        names = self.rar.namelist()
        self.assertEqual(names, ['test_file.txt'])

    def test_testrar(self):
        self.assertIsNone(self.rar.testrar())

    def test_getinfo(self):
        info = self.rar.getinfo('test_file.txt')
        self.assertIsInstance(info, RarInfo)
        self.assertEqual(info.filename, 'test_file.txt')
        self.assertEqual(info.file_size, 17)

    def test_getinfo_no_file(self):
        with self.assertRaises(KeyError) as ctx:
            self.rar.getinfo('foo.txt')

        self.assertEqual(
            ctx.exception.message,
            "There is no item named 'foo.txt' in the archive")

    def test_infolist(self):
        infolist = self.rar.infolist()
        self.assertEqual(len(infolist), 1)
        self.assertEqual(infolist[0], self.rar.getinfo('test_file.txt'))

    def test_extract(self):
        dest_dir = tempfile.gettempdir()
        path = self.rar.extract('test_file.txt', path=dest_dir)
        self.addCleanup(os.remove, path)
        with open(path, 'r') as extracted_file:
            extracted_data = extracted_file.read()
            self.assertEqual(extracted_data, "This is for test.")

    def test_extractall(self):
        dest_dir = tempfile.gettempdir()
        self.rar.extractall(path=dest_dir)
        extracted_path = os.path.join(dest_dir, 'test_file.txt')
        self.addCleanup(os.remove, extracted_path)
        with open(extracted_path, 'r') as extracted_file:
            extracted_data = extracted_file.read()
            self.assertEqual(extracted_data, "This is for test.")


class TestPasswordRarFile(TestRarFile):

    def _open_rarfile(self):
        rar_filename = os.path.join(TESTS_DIR, 'test_password.rar')
        rar = RarFile(rar_filename, pwd='testing')
        return rar


class TestRarSetPasswordFile(TestRarFile):

    def _open_rarfile(self):
        rar_filename = os.path.join(TESTS_DIR, 'test_password.rar')
        rar = RarFile(rar_filename)
        rar.setpassword('testing')
        return rar


class TestCorruptedRar(TestRarFile):

    def _open_rarfile(self):
        rar_filename = os.path.join(TESTS_DIR, 'test_corrupted.rar')
        rar = RarFile(rar_filename)
        rar.setpassword('testing')
        return rar

    def test_testrar(self):
        self.assertEqual(self.rar.testrar(), 'test_file.txt')

    def test_extract(self):
        with self.assertRaises(BadRarFile):
            super(TestCorruptedRar, self).test_extract()

    def test_extractall(self):
        with self.assertRaises(BadRarFile):
            super(TestCorruptedRar, self).test_extractall()



# -*- coding: utf-8 -*-

# Copyright (C) 2012  Matias Bordese
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, find_packages

version = '0.1'

setup(name='unrar',
      version=version,
      description="Wrapper for UnRAR library, ctypes-based.",
      keywords='unrar ctypes',
      author='Matias Bordese',
      author_email='mbordese@gmail.com',
      url='http://github.com/matiasb/python-unrar',
      license='GPL-3',
      packages=find_packages(),
     )



# -*- coding: utf-8 -*-

# Copyright (C) 2012  Matias Bordese
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import ctypes
import os
import platform

from ctypes.util import find_library

from unrar import constants


__all__ = ["RAROpenArchiveDataEx", "RARHeaderDataEx", "RAROpenArchiveEx",
           "RARCloseArchive", "RARReadHeaderEx", "RARProcessFile",
           "RARSetPassword", "RARGetDllVersion", "dostime_to_timetuple"]


lib_path = os.environ.get('UNRAR_LIB_PATH', None)

# find and load unrar library
unrarlib = None
if platform.system() == 'Windows':
    from ctypes.wintypes import HANDLE as WIN_HANDLE
    HANDLE = WIN_HANDLE
    lib_path = lib_path or find_library("unrar.dll")
    if lib_path:
        unrarlib = ctypes.WinDLL(lib_path)
else:
    # assume unix
    HANDLE = ctypes.c_void_p
    lib_path = lib_path or find_library("unrar")
    if lib_path:
        unrarlib = ctypes.cdll.LoadLibrary(lib_path)

if unrarlib is None:
    raise LookupError("Couldn't find path to unrar library.")


def dostime_to_timetuple(dostime):
    """Convert a RAR archive member DOS time to a Python time tuple."""
    dostime = dostime >> 16
    dostime = dostime & 0xffff
    day = dostime & 0x1f
    month = (dostime >> 5) & 0xf
    year = 1980 + (dostime >> 9)
    second = 2 * (dostime & 0x1f)
    minute = (dostime >> 5) & 0x3f
    hour = dostime >> 11
    return (year, month, day, hour, minute, second)


class UnrarException(Exception):
    """Lib errors exception."""


class _Structure(ctypes.Structure):
    """Customized ctypes Structure base class."""

    def __repr__(self):
        """Print the structure fields."""
        res = []
        for field in self._fields_:
            field_value = repr(getattr(self, field[0]))
            res.append('%s=%s' % (field[0], field_value))
        return self.__class__.__name__ + '(' + ','.join(res) + ')'


class RAROpenArchiveDataEx(_Structure):
    """Rar compressed file structure."""
    _fields_ = [
                ('ArcName', ctypes.c_char_p),
                ('ArcNameW', ctypes.c_wchar_p),
                ('OpenMode', ctypes.c_uint),
                ('OpenResult', ctypes.c_uint),
                ('_CmtBuf', ctypes.c_void_p),
                ('CmtBufSize', ctypes.c_uint),
                ('CmtSize', ctypes.c_uint),
                ('CmtState', ctypes.c_uint),
                ('Flags', ctypes.c_uint),
                ('Reserved', ctypes.c_uint * 32),
               ]

    def __init__(self, filename, mode=constants.RAR_OM_LIST):
        # comments buffer, max 64kb
        self.CmtBuf = ctypes.create_string_buffer('', 64 * 1024)
        super(RAROpenArchiveDataEx, self).__init__(
            ArcName=None,
            ArcNameW=filename, OpenMode=mode,
            _CmtBuf=ctypes.addressof(self.CmtBuf),
            CmtBufSize=ctypes.sizeof(self.CmtBuf))

    def __str__(self):
        return self.ArcNameW


class RARHeaderDataEx(_Structure):
    """Rar file header structure."""
    _fields_ = [
                ('ArcName', ctypes.c_char * 1024),
                ('ArcNameW', ctypes.c_wchar * 1024),
                ('FileName', ctypes.c_char * 1024),
                ('FileNameW', ctypes.c_wchar * 1024),
                ('Flags', ctypes.c_uint),
                ('PackSize', ctypes.c_uint),
                ('PackSizeHigh', ctypes.c_uint),
                ('UnpSize', ctypes.c_uint),
                ('UnpSizeHigh', ctypes.c_uint),
                ('HostOS', ctypes.c_uint),
                ('FileCRC', ctypes.c_uint),
                ('FileTime', ctypes.c_uint),
                ('UnpVer', ctypes.c_uint),
                ('Method', ctypes.c_uint),
                ('FileAttr', ctypes.c_uint),
                ('_CmtBuf', ctypes.c_void_p),
                ('CmtBufSize', ctypes.c_uint),
                ('CmtSize', ctypes.c_uint),
                ('CmtState', ctypes.c_uint),
                ('Reserved', ctypes.c_uint * 1024),
               ]

    def __init__(self):
        # comments buffer, max 64kb
        self.CmtBuf = ctypes.create_string_buffer('', 64 * 1024)
        super(RARHeaderDataEx, self).__init__(
            _CmtBuf=ctypes.addressof(self.CmtBuf),
            CmtBufSize=ctypes.sizeof(self.CmtBuf))

    def __str__(self):
        return self.FileNameW


def _c_func(func, restype, argtypes, errcheck=None):
    """Wrap c function setting prototype."""
    func.restype = restype
    func.argtypes = argtypes
    if errcheck is not None:
        func.errcheck = errcheck
    return func


def _check_open_result(res, func, args):
    if res is None:
        raise UnrarException("Archive open error")
    # res is the archive handle
    return res


def _check_readheader_result(res, func, args):
    if res == constants.ERAR_BAD_DATA:
        raise UnrarException("Bad header data.")
    # res == SUCCESS | res == ERAR_END_ARCHIVE
    return res


def _check_process_result(res, func, args):
    if res == constants.ERAR_ECLOSE:
        raise UnrarException("File close error")
    elif res == constants.ERAR_BAD_DATA:
        raise UnrarException("File CRC error")
    elif res == constants.ERAR_BAD_ARCHIVE:
        raise UnrarException("Not valid RAR archive")
    elif res == constants.ERAR_UNKNOWN_FORMAT:
        raise UnrarException("Unknown archive format")
    elif res == constants.ERAR_EOPEN:
        raise UnrarException("Volume open error")
    elif res == constants.ERAR_ECREATE:
        raise UnrarException("File create error")
    elif res == constants.ERAR_EREAD:
        raise UnrarException("Read error")
    elif res == constants.ERAR_EWRITE:
        raise UnrarException("Write error")
    # res == SUCCESS | res == ERAR_END_ARCHIVE
    return res


def _check_close_result(res, func, args):
    if res == constants.ERAR_ECLOSE:
        raise UnrarException("Archive close error")
    # res == SUCCESS
    return res


# Return library API version.
RARGetDllVersion = _c_func(unrarlib.RARGetDllVersion, ctypes.c_int, [])


# Open RAR archive and allocate memory structures (unicode)
RAROpenArchiveEx = _c_func(unrarlib.RAROpenArchiveEx, HANDLE,
                           [ctypes.POINTER(RAROpenArchiveDataEx)],
                           _check_open_result)


# Set a password to decrypt files.
RARSetPassword = _c_func(unrarlib.RARSetPassword, ctypes.c_int,
                         [HANDLE, ctypes.c_char_p])


# Read header of file in archive (unicode).
RARReadHeaderEx = _c_func(unrarlib.RARReadHeaderEx, ctypes.c_int,
                          [HANDLE, ctypes.POINTER(RARHeaderDataEx)],
                          _check_readheader_result)


# Performs action and moves the current position in the archive to
# the next file. Extract or test the current file from the archive
# opened in RAR_OM_EXTRACT mode. If the mode RAR_OM_LIST is set,
# then a call to this function will simply skip the archive position
# to the next file. (unicode)
RARProcessFile = _c_func(unrarlib.RARProcessFile, ctypes.c_int,
                         [HANDLE, ctypes.c_int, ctypes.c_char_p,
                          ctypes.c_char_p], _check_process_result)


# Close RAR archive and release allocated memory. It must be called when
# archive processing is finished, even if the archive processing was stopped
# due to an error.
RARCloseArchive = _c_func(unrarlib.RARCloseArchive, ctypes.c_int, [HANDLE],
                          _check_close_result)
